@@ -1,9 +1,8 @@
 import logging
 import threading
 from collections import namedtuple
-from typing import Callable, Dict, Optional, Type
+from typing import Callable, Dict, Optional, Type, Union, get_origin, get_args
 
-import sentry_sdk
 import zmq
 from pydantic import ValidationError, BaseModel
 from pydantic.main import ROOT_KEY
@@ -124,10 +123,23 @@ class RPCServer(object):
         self.thread.start()
         return self.thread
 
+    def _response_to_dict(self, response):
+        data = response.dict()
+        if response.__custom_root_type__:
+            data = data[ROOT_KEY]
+        return data
+
     def process_rpc(self, channel, message: RPCRequest) -> Optional[Dict]:
         if self.resource and message.resource and self.resource != message.resource:
+            self.log.debug("a rpc was not processed because the resource did not mach. "
+                           "my resource: %s, resource specified in rpc: %s", self.resource, message.resource)
             return None
 
+        if self.log.level <= logging.DEBUG:
+            self.log.debug("received RPC channel: %s, type: %s, request: %s", channel, message.type, message.request)
+        else:
+            self.log.info("received RPC channel: %s, type: %s", channel, message.type)
+
         handler: HandlerEntry = self.find_handler(channel, message.type)
         if handler:
             try:
@@ -144,22 +156,18 @@ class RPCServer(object):
 
             try:
                 response = handler.handler(request, message)
-                if isinstance(response, dict):
-                    return handler.response_class(**response).dict()
-                elif response is None:
-                    return {}  # None is used for when we do not handle the function, so we convert to dict here
-                elif isinstance(response, handler.response_class):
-                    data = response.dict()
-                    if response.__custom_root_type__:
-                        data = data[ROOT_KEY]
-                    return data
+                if isinstance(response, handler.response_class):
+                    # shortcut prevent serialization and de-serialization if the correct response
+                    # is already returned
+                    return self._response_to_dict(response)
                 else:
-                    raise Exception("rpc handler must return None, a dict or a instance of its response_class")
+                    response = handler.response_class.parse_obj(response or {})
+                    return self._response_to_dict(response)
             except Exception as e:
-                self.log.exception("exception in handler {}:{}".format(channel, type))
+                self.log.exception("exception in handler {}:{}".format(channel, message.type))
                 return {"status": "handler_error", "exception": str(e)}
         else:
-            self.log.warning("no handler found for {}:{}".format(channel, type))
+            self.log.warning("no handler found for {}:{}".format(channel, message.type))
             return None
 
     def run(self):
