@@ -438,27 +438,33 @@ class ZMQTestMixin(TestCase):
         super(ZMQTestMixin, self).tearDown()
         Schema.to_file()
 
+    def rpc(self, channel: str, type: str, request, timeout=3, receive_response=True, resource=None) \
+            -> Union[str, dict]:
+        """
+        :return: the rpc id, which can be used with `receive_rpc` to receive the response async if `receive_response` is set to False
+        :return: the rpc response as a dictionary if `receive_response` is set to True
+        """
+        rpc_id = str(uuid4())
+        rpc_request = _TestRPCRequest(name="testclient", type=type, rpc_id=rpc_id,
+                                      resource=resource, request=request)
+        self.zmq_client.send(f"rpc-req-{channel}".encode("ascii"), rpc_request)
+        if receive_response:
+            return self.receive_rpc_response(channel, rpc_id, timeout=timeout)
+        else:
+            return rpc_id
 
-    def rpc(self, channel: str, type: str, request, timeout=3, receive_response=True, resource=None) -> Optional[dict]:
+    def receive_rpc_response(self, channel: str, rpc_id: str, timeout=3) -> dict:
         response_topic = f"rpc-rep-{channel}".encode("ascii")
         if response_topic not in self.zmq_client.subscribed_topics:
             raise DeveloperError(
                 "you have to list the rpc channels you want to use beforehand. please set:\n"
                 f'  > rpc_channels = ["{channel}"] in your test class '
             )
-
-        rpc_id = str(uuid4())
-        rpc_request = _TestRPCRequest(name="testclient", type=type, rpc_id=rpc_id,
-                                      resource=resource, request=request)
-        self.zmq_client.send(f"rpc-req-{channel}".encode("ascii"), rpc_request)
-        if receive_response:
-            _, response = self.zmq_client.wait_for_message(
-                filter=lambda topic, message: topic == response_topic and message.get("rpc_id") == rpc_id,
-                timeout=timeout
-            )
-            return response
-        else:
-            return None
+        _, response = self.zmq_client.wait_for_message(
+            filter=lambda topic, message: topic == response_topic and message.get("rpc_id") == rpc_id,
+            timeout=timeout
+        )
+        return response
 
     def assertRPCFailure(self, response: dict, expected_status="error"):
         self.assertEqual(expected_status, response.get("response", {}).get("status"),
